# encoding: utf-8
require 'redis'
require 'yell'
require 'nokogiri'

unless defined? ActiveSupport
  class String
    def to_query(key)
      require 'cgi' unless defined?(CGI) && defined?(CGI::escape)
      "#{CGI.escape(key.to_s)}=#{CGI.escape(to_s)}"
    end
  end

  class Hash
    def to_query(namespace = nil)
      collect do |key, value|
        value.to_s.to_query(namespace ? "#{namespace}[#{key}]" : key)
      end.compact.sort! * '&'
    end
  end
end

require 'wechat_client/cache_store/redis_store'

module WechatClient
  class << self
    attr_reader :options, :logger, :client
  end

  @options = {}
  @logger = Yell.new :datefile, 'wechat_client.log'
  @cache_client  = Redis.new(host: '127.0.0.1', port: '6379', db: '0')

  def self.cache
    if @cache_client && client.options.auto_cache
      klass = "WechatClient::CacheStore::" + @cache_client.class.name + "Store"
      @_cache_ ||= const_get(klass).new(@cache_client)
    end
  end

  def self.client
    @client ||= WechatClient::Client.new @options
  end
end

require 'wechat_client/client'
require 'wechat_client/o_auth'

