module WechatClient
  class OAuth
    attr_reader :code, :scope, :expires_in, :unionid, :access_token, :refresh_token, :openid

    def initialize options
      set_valuables options[:code]
    end

    def auth
      if @scope == "snsapi_base"
        WechatClient.client.base_info openid: @openid
      else
        WechatClient.client.auth_info access_token: @access_token, openid: @openid
      end
    end

    def refresh_auth
      WechatClient.client.refresh_auth_access_token refresh_token: @refresh_token
    end

    def valid?
      WechatClient.client.auth_access_token_validation access_token: @access_token,\
       openid: @openid do |json|
        json["errmsg"].downcase == "ok"
      end
    end

    def self.generate_auth_link url, state = "", scope = "snsapi_userinfo"
      params = {
        appid: WechatClient.client.options.app_id,
        redirect_uri: url,
        response_type: "code",
        scope: scope,
        state: state
      }
      "https://open.weixin.qq.com/connect/oauth2/authorize?#{params.to_query}#wechat_redirect"
    end

    private

    def set_valuables code
      @code = code
      WechatClient.client.auth_access_token code: @code do |json|
        json.each do |k, v|
          instance_variable_set "@#{k}", v
        end
      end
    end
  end
end
